/*
 * CatalogImplTest.java
 * JUnit based test
 *
 * Created on 27. říjen 2004, 12:16
 */

package cz.muni.fi.pa165.catalog;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import junit.framework.TestCase;


/**
 *
 * @author petr
 */
public class CatalogImplTest extends TestCase {
    
    public CatalogImplTest(String testName) {
        super(testName);
    }
    
    private static final String URL_PROPERTY = "cz.muni.fi.pa165.catalog.jdbcUrl";
    private static final String DEFAULT_URL = "jdbc:hsqldb:.";
    
    protected Catalog catalog;
    
    protected void setUp() throws java.lang.Exception {
        System.setProperty(URL_PROPERTY, DEFAULT_URL);
        catalog = new CatalogImpl();
    }
    
    protected void tearDown() throws java.lang.Exception {
        if (catalog != null) {
            catalog.close();
        }
    }
    
    
    public static junit.framework.Test suite() {
        junit.framework.TestSuite suite = new junit.framework.TestSuite(CatalogImplTest.class);
        return suite;
    }
    
    // TODO add test methods here. The name must begin with 'test'. For example:
    
    public void testCreateGroup() throws CatalogException {
        Group rootGroup = catalog.getRootGroup();
        assertNotNull("root group is nul",rootGroup);
        
        Group newGroup = catalog.newGroup(rootGroup,"test group");
        assertNotNull(newGroup);
        assertEquals("test group",newGroup.getName());
        
        Product newProduct = catalog.newProduct(newGroup,"test product", 22.5);
        assertNotNull(rootGroup);
        assertEquals(22.5,newProduct.getPrice());
        assertEquals("test product",newProduct.getName());
        
        assertTrue("The Group does not contain product that is added to it",newGroup.getProducts().contains(newProduct));
        assertTrue("The Group does not contain group that is added to it",rootGroup.getChildGroups().contains(newGroup));
    }
    
    private static class FakeGroup implements Group {
        public int getOid() { return 0; }
        public String getName() { return ""; }
        public void setName(String name) { };
        public List<Product> getProducts() { return new ArrayList<Product>(); }
        public List<Group> getChildGroups() { return new ArrayList<Group>(); }
    }
    
    public void testBadParentGroup() throws CatalogException {
        Group rootGroup = new FakeGroup();
        try {
            catalog.newGroup(rootGroup,"test group");
            fail("The Catalog allows to create new group with parent group that is not from this catalog");
        }
        catch (IllegalArgumentException ex) {
            // all is OK
        }
        try {
            catalog.newProduct(rootGroup,"test group", 0);
            fail("The Catalog allows to create new product with parent group that is not from this catalog");
        }
        catch (IllegalArgumentException ex) {
            // all is OK
        }
    }
    
    public void testBadPrice() throws CatalogException {
        Group rootGroup = catalog.getRootGroup();
        try {
            catalog.newProduct(rootGroup,"test product", -1);
            fail("The Catalog allows to create new product with negative price");
        }
        catch (IllegalArgumentException ex) {
            // all is OK
        }
        
        try {
            catalog.newProduct(rootGroup,"test product", 10).setPrice(-1);
            fail("The Catalog allows to change product price to negative value");
        }
        catch (IllegalArgumentException ex) {
            // all is OK
        }
    }
    
    public void testReadOnlyLists() throws CatalogException {
        Group rootGroup = catalog.getRootGroup();
        Group newGroup = catalog.newGroup(rootGroup,"test group");
        Product newProduct = catalog.newProduct(newGroup,"test product", 22.5);
        
        try {
            rootGroup.getChildGroups().add(newGroup);
            fail("The Group allows to modify list of groups returned by getChildgroups()");
        }
        catch (UnsupportedOperationException ex) {
            // all is ok
        }
        try {
            rootGroup.getProducts().add(newProduct);
            fail("The Group allows to modify list of products returned by getProducts()");
        }
        catch (UnsupportedOperationException ex) {
            // all is ok
        }
    }
    
    public void testNoURLProperty() {
        System.clearProperty(URL_PROPERTY);
        try {
            new CatalogImpl();
            fail("It is possible to create a Catalog wihout set the " + URL_PROPERTY + " property");
        }
        catch (CatalogException ex) {
            // All seems to be OK
        }
    }

    public void testClosedCatalog() throws CatalogException {
        Group rootGroup = catalog.getRootGroup();
        catalog.close();
        try {
            catalog.getRootGroup();
            catalog = null;
            fail("It is possible to work with closed catalog (getRootGroup)");
        }
        catch (IllegalStateException ex) {
            // All is ok
        }

        try {
            catalog.newProduct(rootGroup,"test",10);
            catalog = null;
            fail("It is possible to work with closed catalog (newProduct)");
        }
        catch (IllegalStateException ex) {
            // All is ok
        }

        try {
            catalog.newGroup(rootGroup,"test");
            catalog = null;
            fail("It is possible to work with closed catalog (newGroup)");
        }
        catch (IllegalStateException ex) {
            // All is ok
        }

        catalog = null;
    }

    public File createTempDir(String prefix) throws IOException {
        // This is an ugly hack :-(
        File result = File.createTempFile(prefix,"");
        result.delete();
        result.mkdir();
        // end of ugly hack
        return result;
    }
    
    private void deleteTempDir(File dir) {
        // This does not work for directories with subdirectories
        File[] files = dir.listFiles();
        for (int i=0;i<files.length;i++) {
            files[i].delete();
        }
        dir.delete();
    }
    
    public void testPersistence() throws CatalogException, IOException {
        
        File testDb = createTempDir("CatalogTest");
        final String PERSISTENT_URL = "jdbc:hsqldb:" + new File(testDb, "test");

        try {

            System.setProperty(URL_PROPERTY, PERSISTENT_URL);
            Catalog catalog = new CatalogImpl();
            Group rootGroup = catalog.getRootGroup();
            // The Catalog should be empty, therefore the next group should be 
            // the first child of the root group
            Group newGroup = catalog.newGroup(rootGroup,"test group");
            Product newProduct = catalog.newProduct(newGroup,"test product", 22.5);
            catalog.close();
            
            Catalog catalog2 = new CatalogImpl();
            Group rootGroup2 = catalog2.getRootGroup();
            assertNotNull(rootGroup2);
            assertEquals("The root group from newly opened catalog is not " +
            "equals to the root group from the previously opened catalog " +
            "(probably the equals method from GroupImpl is not " +
            "corrrectly implemented)",rootGroup,rootGroup2);

            assertEquals("The root group from newly opened catalog has " +
            "different hashcode then the root group from the previously " +
            "opened one (probably the hashcode method from GroupImpl is " +
            "not corrrectly implemented)",
            rootGroup.hashCode(),rootGroup2.hashCode());         

            assertTrue("The root group does not contain group that is added to it",
            rootGroup2.getChildGroups().contains(newGroup));
            Group newGroup2 = rootGroup2.getChildGroups().get(0);
            assertNotNull(newGroup2);
            assertEquals("The first child group of root group is not equals to " +
            "the group that was added as only child of root group from the " +
            "previously opened catalog (catalog was not empty before this test, the " +
            "equals method is not correctly implemented or the root group " +
            "contains other child groups).",newGroup,newGroup2);

            assertTrue("The new group does not contain product that was added " +
            "to it in previously opened catalog (probably the equals method in " +
            "class ProductImpl is not corectly implemented)",
            newGroup2.getProducts().contains(newProduct));
            
            Product newProduct2 = newGroup2.getProducts().get(0);
            assertEquals("The first product from new group is not equals to " +
            "the product that was added as only product of this group in the " +
            "previously opened catalog (probably the equals method in " +
            "class ProductImpl is not corectly implemented).",
            newProduct,newProduct2);

            assertEquals("The product hase different hashcode than " +
            "the same product from previously opened catalog (probably " +
            "the hashcode method of class ProductImpl is not " +
            "correctly implemented ).",
            newProduct.hashCode(),newProduct2.hashCode());
            
            assertEquals(22.5,newProduct2.getPrice());
            assertEquals("test product",newProduct2.getName());
            assertEquals("test group",newGroup2.getName());
            
        }
        finally {
            deleteTempDir(testDb);
        }

    }
}
